import {
  Type,
} from '@overflow/commons/core/type';

import * as TypeUtil from '@overflow/commons/core/type/util';

import * as Constants from './constants';
import {
  Annotation,
} from './annotation';

export class Decorator {
  public static create = (AnnotationClass: Type<Annotation>) => {

    return (...handlerArgs: any[]) => {
      let annotation: Annotation = new AnnotationClass(...handlerArgs);
      let name: string = AnnotationClass.name;

      return (...decoratorArgs: any[]) => {
        let decoratorType: Constants.DecoratorType = Decorator._detectDecoratorType(name, annotation, decoratorArgs);
        // let type = typeof decoratorArgs[0] === 'function' ? decoratorArgs[0].prototype : decoratorArgs[0];
        // let clazz = TypeUtil.getClass(decoratorArgs[0]);

        switch(decoratorType) {
        case Constants.DecoratorType.CLASS:
          return annotation.classDecorator.call(annotation, decoratorArgs[0]);
        case Constants.DecoratorType.PROPERTY:
          return annotation.propertyDecorator.call(annotation, decoratorArgs[0], decoratorArgs[1]);
        case Constants.DecoratorType.METHOD:
          return annotation.methodDecorator.call(annotation, decoratorArgs[0], decoratorArgs[1], decoratorArgs[2]);
        case Constants.DecoratorType.PARAMETER:
          return annotation.parameterDecorator.call(annotation, decoratorArgs[0], decoratorArgs[1], decoratorArgs[2]);
        default:
        }
      };
    };
  }

  private static _detectDecoratorType(name: string, annotation: any, args: any[]): Constants.DecoratorType {
    let params = [];
    for (let i = 0; i < args.length; i++) {
      if (args[i]) {
        params.push(args[i]);
      }
    }
    const paramCount = params.length;
    let decoratorType: Constants.DecoratorType;

    if (1 === paramCount) {
      if (typeof(annotation.classDecorator) === 'undefined') {
        throw new Error(`Cannot apply @${name} decorator on Class.`);
      }
      decoratorType = Constants.DecoratorType.CLASS;
    } else if (2 === paramCount) {
      if (typeof(annotation.propertyDecorator) === 'undefined') {
        throw new Error(`Cannot apply @${name} decorator on Property.`);
      }
      decoratorType = Constants.DecoratorType.PROPERTY;
    } else if (3 === paramCount) {
      if(typeof args[2] === 'number') {
        if (typeof(annotation.parameterDecorator) === 'undefined') {
          throw new Error(`Cannot apply @${name} decorator on Parameter.`);
        }
        decoratorType = Constants.DecoratorType.PARAMETER;
      } else {
        if (typeof(annotation.methodDecorator) === 'undefined') {
          throw new Error(`Cannot apply @${name} decorator on Method.`);
        }
        decoratorType = Constants.DecoratorType.METHOD;
      }
    } else {
      throw new Error(`@${name} decorator is not valid here!`);
    }

    return decoratorType;
  }
}
