import * as React from 'react';
import * as ReactDOM from 'react-dom';

import {
  applyMiddleware,
  compose,
  createStore,
  GenericStoreEnhancer,
  Middleware,
  Store,
} from 'redux';

import {
  Provider,
} from 'react-redux';

import {
  ConnectedRouter,
  routerMiddleware,
} from 'react-router-redux';

import {
  createHashHistory,
  History,
} from 'history';

import {
  AppContainer,
} from 'react-hot-loader';

import config from './config';
import WebAppReducer from '@overflow/webapp/redux/webapp_reducer';

// declare let module: { hot: any };

declare global {
  interface Window {
    devToolsExtension: () => any;
  }
  const process: any;
  const module: { hot: any };
}

class WebAppApplication {
  private static isProduction:boolean = process.env.NODE_ENV === 'production' ? true : false;
  private static useReduxDevTools:boolean = window.devToolsExtension && !WebAppApplication.isProduction ? true : false;

  private container: HTMLElement;
  private store: Store<any>;
  private history: History;

  public constructor() {
    this.container = document.getElementById('appContainer');
    this.history = createHashHistory();
  }

  public async run(): Promise<void> {
    try {
      this.renderLoading();
      let reducer: WebAppReducer = WebAppReducer.getInstance();
      this.store = createStore(reducer.reducer, config.redux.state);

      this.renderApp();
    } catch (e) {
      console.error(e);
    }
  }

  private renderLoading(): void {
    ReactDOM.render(
        <div style={{
          width: '100vw',
          height: '100vh',
          backgroundColor: 'white',
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center',
          }}>
            <h1>Loading...</h1>
        </div>,
        this.container,
    );
  }

  private renderError(e: Error): void {
    ReactDOM.render(
        <div style={{
          width: '100vw',
          height: '100vh',
          backgroundColor: 'white',
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center',
          }}>
            <h1>{e.message}</h1>
        </div>,
        this.container,
    );
  }

  private renderApp(): void {
    WebAppApplication.isProduction ? this.renderProduction() : this.rederDevelopment();
  }
  private renderProduction(): void {
    ReactDOM.render(
      <Provider store={this.store}>
        <ConnectedRouter history={this.history}>

        </ConnectedRouter>
      </Provider>,
      this.container,
    );
  }

  private rederDevelopment(): void {
    if (module.hot) {
      module.hot.accept('./pages/webapp', async () => {
        const NextApp = (await import('./pages/webapp')).default;
        ReactDOM.render(
          <AppContainer>
            <Provider store={this.store}>
              <ConnectedRouter history={this.history}>
                <NextApp />
              </ConnectedRouter>
            </Provider>
          </AppContainer>
          ,
          this.container,
        );
      });
    } else {
      ReactDOM.render(
        <AppContainer>
          <Provider store={this.store}>
            <ConnectedRouter history={this.history}>

            </ConnectedRouter>
          </Provider>
        </AppContainer>
        ,
        this.container,
      );
    }

  }



  public static main(): void {
    let webapp = new WebAppApplication();
    webapp.run();
  }
}

WebAppApplication.main();
